Option Explicit On

Public Class frmEditDialog

    Private Const WM_CUT = &H300
    Private Const WM_COPY = &H301
    Private Const WM_PASTE = &H302

    Private _objEditDialog As clsEditDialog = Nothing

    Private Sub _updateInfo()

        Dim lineIndex As Integer = txtEdit.GetLineFromCharIndex(txtEdit.SelectionStart) + 1
        Dim colIndex As Integer = (txtEdit.SelectionStart - txtEdit.GetFirstCharIndexFromLine(lineIndex - 1)) + 1

        lblInfo.Text = "   Ln " + lineIndex.ToString + ", Col " + colIndex.ToString

    End Sub

    Public Sub New(ByVal objEditDialog As clsEditDialog, _
        Optional ByVal caption As String = Nothing)

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        _objEditDialog = objEditDialog

        If Not (caption Is Nothing) Then Me.Text = caption

        txtEdit.Text = _objEditDialog.InputText

    End Sub

    Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click

        Me.Close()

    End Sub

    Private Sub btnSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSave.Click

        _objEditDialog.OutputText = txtEdit.Text
        _objEditDialog = Nothing
        Me.Close()

    End Sub

    Private Sub frmEditDialog_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        If e.CloseReason = CloseReason.UserClosing And _
            Not (_objEditDialog Is Nothing) Then _
            _objEditDialog.Aborted = True

    End Sub

    Private Sub txtEdit_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtEdit.Click

        _updateInfo()

    End Sub

    Private Sub txtEdit_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles txtEdit.KeyUp

        _updateInfo()

        If e.KeyCode = Keys.F3 Then btnFind_Click(sender, Nothing)

    End Sub

    Private Sub txtEdit_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtEdit.TextChanged

        _updateInfo()

    End Sub

    Private Sub btnCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCut.Click

        With New clsWindowObject(txtEdit.Handle)
            .SendMsg(WM_CUT, 0, 0)
        End With

    End Sub

    Private Sub btnCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCopy.Click

        With New clsWindowObject(txtEdit.Handle)
            .SendMsg(WM_COPY, 0, 0)
        End With

    End Sub

    Private Sub btnPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPaste.Click

        With New clsWindowObject(txtEdit.Handle)
            .SendMsg(WM_PASTE, 0, 0)
        End With

    End Sub

    Private Sub btnFind_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFind.Click

        Static lastFindText As String = ""
        Dim findText As String = ""

        If e Is Nothing And lastFindText <> "" Then
            findText = lastFindText
        Else
            findText = InputBox("Enter the text to find:", Application.ProductName, lastFindText)
        End If

        If findText <> "" Then

            Dim findIndex As Integer = txtEdit.Text.IndexOf(findText, txtEdit.SelectionStart + txtEdit.SelectionLength)

            If findIndex >= 0 Then

                txtEdit.SelectionStart = findIndex
                txtEdit.SelectionLength = findText.Length

            Else

                MessageBox.Show("Cannot find """ + findText + """", Application.ProductName, _
                    MessageBoxButtons.OK, MessageBoxIcon.Information)

            End If

        End If

        lastFindText = findText

    End Sub

End Class