Option Explicit On

Imports System.IO
Imports System.Net
Imports Be.Windows.Forms

Public Class frmIPSpy

    Private _udpClient As clsUdpClient = Nothing
    Private _udpServer As clsUdpServer = Nothing
    Private _tcpClient As clsTcpClient = Nothing
    Private _tcpServer As clsTcpServer = Nothing
    Private _tcpServerClient As clsTcpClient = Nothing

    Private _clientIn As DynamicByteProvider = Nothing
    Private _clientOut As DynamicByteProvider = Nothing
    Private _serverIn As DynamicByteProvider = Nothing
    Private _serverOut As DynamicByteProvider = Nothing

    Private clientText As String = ""
    Private clientUpdate As DateTime = Nothing
    Private serverText As String = ""
    Private serverUpdate As DateTime = Nothing

    Private Const BytesPerLine As Integer = 8
    Private Const UpdateExpires As Integer = 5 ' seconds
    Private Const DefaultMaxClients As Integer = 1

    Public Function UdpClientDataRecvByteCallback(ByVal byteData() As Byte) As Byte()

        If Me.InvokeRequired Then
            Return Me.Invoke(New clsUdpClient.UdpClientDataRecvByteFunc(AddressOf UdpClientDataRecvByteCallback), byteData)
        Else

            _clientIn.Bytes.AddRange(byteData)
            hexClientIn.Refresh()

            lblClient.Text = "Client: Recv " + byteData.Length.ToString + " byte(s)"
            clientUpdate = Now()

            Return Array.CreateInstance(GetType(Byte), 0)

        End If

    End Function

    Public Function UdpServerDataRecvByteCallback(ByVal byteData() As Byte) As Byte()

        If Me.InvokeRequired Then
            Return Me.Invoke(New clsUdpServer.UdpServerDataRecvByteFunc(AddressOf UdpServerDataRecvByteCallback), byteData)
        Else

            _serverIn.Bytes.AddRange(byteData)
            hexServerIn.Refresh()

            lblServer.Text = "Server: Recv " + byteData.Length.ToString + " byte(s)"
            serverUpdate = Now()

            Return Array.CreateInstance(GetType(Byte), 0)

        End If

    End Function

    Public Function TcpClientDataRecvByteCallback(ByVal byteData() As Byte) As Byte()

        If Me.InvokeRequired Then
            Return Me.Invoke(New clsTcpClient.TcpClientDataRecvByteFunc(AddressOf TcpClientDataRecvByteCallback), byteData)
        Else

            _clientIn.Bytes.AddRange(byteData)
            hexClientIn.Refresh()

            lblClient.Text = "Client: Recv " + byteData.Length.ToString + " byte(s)"
            clientUpdate = Now()

            Return Array.CreateInstance(GetType(Byte), 0)

        End If

    End Function

    Public Function TcpServerDataRecvByteCallback(ByVal byteData() As Byte) As Byte()

        If Me.InvokeRequired Then
            Return Me.Invoke(New clsTcpClient.TcpClientDataRecvByteFunc(AddressOf TcpServerDataRecvByteCallback), byteData)
        Else

            _serverIn.Bytes.AddRange(byteData)
            hexServerIn.Refresh()

            lblServer.Text = "Server: Recv " + byteData.Length.ToString + " byte(s)"
            serverUpdate = Now()

            Return Array.CreateInstance(GetType(Byte), 0)

        End If

    End Function

    Public Sub ServerNewClientCallback(ByVal newClient As clsTcpClient)

        If Me.InvokeRequired Then
            Me.Invoke(New clsTcpServer.TcpServerNewClientSub(AddressOf ServerNewClientCallback), newClient)
        Else

            _tcpServerClient = newClient
            _tcpServerClient.StartListen(New clsTcpClient.TcpClientDataRecvByteFunc(AddressOf TcpServerDataRecvByteCallback))

            serverText = "Server: Connection from " + newClient.ToString
            lblServer.Text = serverText
            serverUpdate = Now()

        End If

    End Sub

    Public Sub ClientDisconnectCallback(ByVal thisClient As clsTcpClient)

        If Me.InvokeRequired Then
            Me.Invoke(New clsTcpClient.TcpClientDisconnectSub(AddressOf ClientDisconnectCallback), thisClient)
        Else

            clientText = "Client: Ready"
            lblClient.Text = "Client: Disconnected from " + thisClient.ToString
            clientUpdate = Now()

            _tcpClient.Dispose()
            _tcpClient = Nothing

        End If

    End Sub

    Public Sub ServerClientDisconnectCallback(ByVal thisClient As clsTcpClient)

        If Me.InvokeRequired Then
            Me.Invoke(New clsTcpClient.TcpClientDisconnectSub(AddressOf ServerClientDisconnectCallback), thisClient)
        Else

            If _tcpServer.Clients.Count = 0 Then
                serverText = "Server: Listening on port " + CStr(Val(txtServerPort.Text))
            Else
                serverText = "Server: " + _tcpServer.Clients.Count.ToString + " connected client(s)"
            End If

            lblServer.Text = "Server: Disconnect from " + thisClient.ToString
            serverUpdate = Now()

            _tcpServerClient.Dispose()
            _tcpServerClient = Nothing

        End If

    End Sub

    Public Sub RequestUserSelection(ByVal objUserSelection As clsUserSelection, _
        Optional ByVal caption As String = Nothing)

        ' interface extension requests user selection
        If Me.InvokeRequired Then
            Me.Invoke(New clsUserSelection.RequestUserSelectionFunc(AddressOf RequestUserSelection), objUserSelection, caption)
        Else

            ' instantiate a new user selection dialog
            With New frmUserSelection(objUserSelection, caption)
                .ShowDialog(Me)
            End With

            ' tell the invoking thread to resume execution
            objUserSelection.Resume()

        End If

    End Sub

    Public Function ChooseIPEntry(ByVal host As String) As IPAddress

        Dim hostIP As IPAddress = Nothing
        Dim hostIPs As IPHostEntry = Dns.GetHostEntry(host)

        If hostIPs.AddressList.Length > 1 Then

            ' create the user selection request
            Dim userRequest As New clsUserSelection

            userRequest.AllowCancel = True

            ' add columns
            userRequest.AddColumn("IP Address")

            ' enumerate each ip entry
            For Each thisIP As IPAddress In hostIPs.AddressList

                userRequest.AddRow(thisIP.ToString)

            Next

            ' show the user selection dialog
            userRequest.Show(New clsUserSelection.RequestUserSelectionFunc(AddressOf RequestUserSelection), _
                "Choose IP Address of Hostname")

            ' wait for the user selection
            userRequest.Wait()

            ' make sure user did not cancel
            If userRequest.retIndex >= 0 Then

                ' get the query file chosen
                hostIP = hostIPs.AddressList(userRequest.retIndex)

            End If

        Else

            hostIP = hostIPs.AddressList(0)

        End If

        Return hostIP

    End Function

    Private Sub frmIPSpy_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Dim _b() As Byte = {}

        _clientIn = New DynamicByteProvider(_b)
        hexClientIn.ByteProvider = _clientIn

        _clientOut = New DynamicByteProvider(_b)
        hexClientOut.ByteProvider = _clientOut

        _serverIn = New DynamicByteProvider(_b)
        hexServerIn.ByteProvider = _serverIn

        _serverOut = New DynamicByteProvider(_b)
        hexServerOut.ByteProvider = _serverOut

        btnClientCmd.Tag = False
        btnServerCmd.Tag = False

    End Sub

    Private Sub frmIPSpy_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize

        If Me.MinimumSize.Width > 0 Then

            Dim cols As Integer = BytesPerLine + Math.Round( _
                (Me.Size.Width - Me.MinimumSize.Width) / 36)

            hexClientOut.BytesPerLine = cols
            hexClientIn.BytesPerLine = cols
            hexServerIn.BytesPerLine = cols
            hexServerOut.BytesPerLine = cols

        End If

    End Sub

    Private Sub btnClientCmd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClientCmd.Click

        If radTCPIP.Checked Then

            If Not btnClientCmd.Tag Then

                btnClientCmd.Tag = True
                btnClientCmd.Text = "&Disconnect"

                txtClientHost.Enabled = False
                txtClientPort.Enabled = False

                radTCPIP.Enabled = False
                radUDP.Enabled = False

                _clientIn.Bytes.Clear()
                hexClientIn.Refresh()

                Try

                    Dim hostIP As IPAddress = Nothing

                    If txtClientHost.Text.Trim <> "" And Val(txtClientPort.Text) > 0 Then

                        If Not IPAddress.TryParse(txtClientHost.Text.Trim, hostIP) Then

                            ' have user choose ip entry for host
                            hostIP = ChooseIPEntry(txtClientHost.Text.Trim)

                        End If

                        If Not (hostIP Is Nothing) Then

                            _tcpClient = New clsTcpClient(hostIP.ToString, Val(txtClientPort.Text), _
                                New clsTcpClient.TcpClientDisconnectSub(AddressOf ClientDisconnectCallback))
                            _tcpClient.StartListen(New clsTcpClient.TcpClientDataRecvByteFunc(AddressOf TcpClientDataRecvByteCallback))

                            clientText = "Client: Connected to " + hostIP.ToString + ":" + CStr(Val(txtClientPort.Text))
                            lblClient.Text = clientText

                        End If

                    Else

                        txtClientHost.Focus()
                        Throw New Exception("Please enter the host or port.")

                    End If

                Catch ex As Exception

                    lblClient.Text = "Client: Ready"
                    clientText = ""

                    txtClientPort.Enabled = True
                    txtClientHost.Enabled = True

                    btnClientCmd.Tag = False
                    btnClientCmd.Text = "&Connect"

                    If Not btnClientCmd.Tag And Not btnServerCmd.Tag Then

                        radTCPIP.Enabled = True
                        radUDP.Enabled = True

                    End If

                    If Not (_tcpClient Is Nothing) Then

                        _tcpClient.Dispose()
                        _tcpClient = Nothing

                    End If

                    MessageBox.Show("Error: " + ex.Message, Application.ProductName, _
                        MessageBoxButtons.OK, MessageBoxIcon.Error)

                End Try

            Else

                lblClient.Text = "Client: Ready"
                clientText = ""

                txtClientPort.Enabled = True
                txtClientHost.Enabled = True

                btnClientCmd.Tag = False
                btnClientCmd.Text = "&Connect"

                If Not btnClientCmd.Tag And Not btnServerCmd.Tag Then

                    radTCPIP.Enabled = True
                    radUDP.Enabled = True

                End If

                If Not (_tcpClient Is Nothing) Then

                    _tcpClient.Dispose()
                    _tcpClient = Nothing

                End If

            End If

        ElseIf radUDP.Checked Then

            If Not btnClientCmd.Tag Then

                btnClientCmd.Tag = True
                btnClientCmd.Text = "&Disconnect"

                txtClientHost.Enabled = False
                txtClientPort.Enabled = False

                radTCPIP.Enabled = False
                radUDP.Enabled = False

                Try

                    Dim hostIP As IPAddress = Nothing

                    If txtClientHost.Text.Trim <> "" And Val(txtClientPort.Text) > 0 Then

                        If Not IPAddress.TryParse(txtClientHost.Text.Trim, hostIP) Then

                            ' have user choose ip entry for host
                            hostIP = ChooseIPEntry(txtClientHost.Text.Trim)

                        End If

                        If Not (hostIP Is Nothing) Then

                            _udpClient = New clsUdpClient(hostIP.ToString, Val(txtClientPort.Text))
                            _udpClient.StartListen(New clsUdpClient.UdpClientDataRecvByteFunc(AddressOf UdpClientDataRecvByteCallback))

                            clientText = "Client: Connected to " + hostIP.ToString + ":" + CStr(Val(txtClientPort.Text))
                            lblClient.Text = clientText

                        End If

                    Else

                        txtClientHost.Focus()
                        Throw New Exception("Please enter the host or port.")

                    End If

                Catch ex As Exception

                    lblClient.Text = "Client: Ready"
                    clientText = ""

                    txtClientPort.Enabled = True
                    txtClientHost.Enabled = True

                    btnClientCmd.Tag = False
                    btnClientCmd.Text = "&Connect"

                    If Not btnClientCmd.Tag And Not btnServerCmd.Tag Then

                        radTCPIP.Enabled = True
                        radUDP.Enabled = True

                    End If

                    If Not (_udpClient Is Nothing) Then

                        _udpClient.Dispose()
                        _udpClient = Nothing

                    End If

                    MessageBox.Show("Error: " + ex.Message, Application.ProductName, _
                        MessageBoxButtons.OK, MessageBoxIcon.Error)

                End Try

            Else

                lblClient.Text = "Client: Ready"
                clientText = ""

                txtClientPort.Enabled = True
                txtClientHost.Enabled = True

                btnClientCmd.Tag = False
                btnClientCmd.Text = "&Connect"

                If Not btnClientCmd.Tag And Not btnServerCmd.Tag Then

                    radTCPIP.Enabled = True
                    radUDP.Enabled = True

                End If

                If Not (_udpClient Is Nothing) Then

                    _udpClient.Dispose()
                    _udpClient = Nothing

                End If

            End If

        End If

    End Sub

    Private Sub btnClientSendData_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClientSendData.Click

        If radTCPIP.Checked Then

            If Not (_tcpClient Is Nothing) Then

                Try

                    Dim buff() As Byte = _clientOut.Bytes.GetBytes()
                    _tcpClient.Send(buff)

                    lblClient.Text = "Client: Sent " + buff.Length.ToString + " byte(s)"
                    clientUpdate = Now()

                Catch ex As Exception

                    MessageBox.Show("Error: " + ex.Message, Application.ProductName, _
                        MessageBoxButtons.OK, MessageBoxIcon.Error)

                End Try

            Else

                MessageBox.Show("Not connected.", Application.ProductName, _
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

            End If

        ElseIf radUDP.Checked Then

            If Not (_udpClient Is Nothing) Then

                Try

                    Dim buff() As Byte = _clientOut.Bytes.GetBytes()
                    _udpClient.Send(buff)

                    lblClient.Text = "Client: Sent " + buff.Length.ToString + " byte(s)"
                    clientUpdate = Now()

                Catch ex As Exception

                    MessageBox.Show("Error: " + ex.Message, Application.ProductName, _
                        MessageBoxButtons.OK, MessageBoxIcon.Error)

                End Try

            Else

                MessageBox.Show("Not connected.", Application.ProductName, _
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

            End If

        End If

    End Sub

    Private Sub btnServerCmd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnServerCmd.Click

        If radTCPIP.Checked Then

            If Not btnServerCmd.Tag Then

                btnServerCmd.Tag = True
                btnServerCmd.Text = "&Disconnect"

                txtServerHost.Enabled = False
                txtServerPort.Enabled = False

                radTCPIP.Enabled = False
                radUDP.Enabled = False

                _serverIn.Bytes.Clear()
                hexServerIn.Refresh()

                Try

                    If Val(txtServerPort.Text) > 0 Then

                        If txtServerHost.Text.Trim <> "" Then

                            _tcpServer = New clsTcpServer(txtServerHost.Text.Trim, Val(txtServerPort.Text), DefaultMaxClients, _
                                New clsTcpServer.TcpServerClientDisconnectSub(AddressOf ServerClientDisconnectCallback))

                        Else

                            _tcpServer = New clsTcpServer(Nothing, Val(txtServerPort.Text), DefaultMaxClients, _
                                New clsTcpServer.TcpServerClientDisconnectSub(AddressOf ServerClientDisconnectCallback))

                        End If

                        _tcpServerClient = Nothing
                        _tcpServer.StartListener(New clsTcpServer.TcpServerNewClientSub(AddressOf ServerNewClientCallback))

                        serverText = "Server: Listening on port " + CStr(Val(txtServerPort.Text))
                        lblServer.Text = serverText

                    Else

                        txtServerPort.Focus()
                        Throw New Exception("Please enter the port.")

                    End If

                Catch ex As Exception

                    lblServer.Text = "Server: Ready"
                    serverText = ""

                    txtServerPort.Enabled = True
                    txtServerHost.Enabled = True

                    btnServerCmd.Tag = False
                    btnServerCmd.Text = "&Listen"

                    If Not btnClientCmd.Tag And Not btnServerCmd.Tag Then

                        radTCPIP.Enabled = True
                        radUDP.Enabled = True

                    End If

                    If Not (_tcpServer Is Nothing) Then

                        _tcpServer.Dispose()
                        _tcpServer = Nothing

                    End If

                    MessageBox.Show("Error: " + ex.Message, Application.ProductName, _
                        MessageBoxButtons.OK, MessageBoxIcon.Error)

                End Try

            Else

                lblServer.Text = "Server: Ready"
                serverText = ""

                txtServerPort.Enabled = True
                txtServerHost.Enabled = True

                btnServerCmd.Tag = False
                btnServerCmd.Text = "&Listen"

                If Not btnClientCmd.Tag And Not btnServerCmd.Tag Then

                    radTCPIP.Enabled = True
                    radUDP.Enabled = True

                End If

                If Not (_tcpServer Is Nothing) Then

                    _tcpServer.Dispose()
                    _tcpServer = Nothing

                End If

            End If

        ElseIf radUDP.Checked Then

            If Not btnServerCmd.Tag Then

                btnServerCmd.Tag = True
                btnServerCmd.Text = "&Disconnect"

                txtServerHost.Enabled = False
                txtServerPort.Enabled = False

                radTCPIP.Enabled = False
                radUDP.Enabled = False

                Try

                    If Val(txtServerPort.Text) > 0 Then

                        If txtServerHost.Text.Trim <> "" Then

                            _udpServer = New clsUdpServer(txtServerHost.Text.Trim, Val(txtServerPort.Text))

                        Else

                            _udpServer = New clsUdpServer(Nothing, Val(txtServerPort.Text))

                        End If

                        _udpServer.StartListen(New clsUdpServer.UdpServerDataRecvByteFunc(AddressOf UdpServerDataRecvByteCallback))

                        serverText = "Server: Binded to port " + CStr(Val(txtServerPort.Text))
                        lblServer.Text = serverText

                    Else

                        txtServerPort.Focus()
                        Throw New Exception("Please enter the port.")

                    End If

                Catch ex As Exception

                    lblServer.Text = "Server: Ready"
                    serverText = ""

                    btnServerCmd.Tag = False
                    btnServerCmd.Text = "&Listen"

                    If Not btnClientCmd.Tag And Not btnServerCmd.Tag Then

                        radTCPIP.Enabled = True
                        radUDP.Enabled = True

                    End If

                    If Not (_udpServer Is Nothing) Then

                        _udpServer.Dispose()
                        _udpServer = Nothing

                    End If

                    MessageBox.Show("Error: " + ex.Message, Application.ProductName, _
                        MessageBoxButtons.OK, MessageBoxIcon.Error)

                End Try

            Else

                lblServer.Text = "Server: Ready"
                serverText = ""

                txtServerPort.Enabled = True
                txtServerHost.Enabled = True

                btnServerCmd.Tag = False
                btnServerCmd.Text = "&Listen"

                If Not btnClientCmd.Tag And Not btnServerCmd.Tag Then

                    radTCPIP.Enabled = True
                    radUDP.Enabled = True

                End If

                If Not (_udpServer Is Nothing) Then

                    _udpServer.Dispose()
                    _udpServer = Nothing

                End If

            End If

        End If

    End Sub

    Private Sub btnServerSendData_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnServerSendData.Click

        If radTCPIP.Checked Then

            If Not (_tcpServer Is Nothing) Then

                If Not (_tcpServerClient Is Nothing) Then

                    Try

                        Dim buff() As Byte = _serverOut.Bytes.GetBytes()
                        _tcpServerClient.Send(buff)

                        lblServer.Text = "Server: Sent " + buff.Length.ToString + " byte(s)"
                        serverUpdate = Now()

                    Catch ex As Exception

                        MessageBox.Show("Error: " + ex.Message, Application.ProductName, _
                            MessageBoxButtons.OK, MessageBoxIcon.Error)

                    End Try

                Else

                    MessageBox.Show("No client connected.", Application.ProductName, _
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

                End If

            Else

                MessageBox.Show("Not connected.", Application.ProductName, _
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

            End If

        ElseIf radUDP.Checked Then

            If Not (_udpServer Is Nothing) Then

                Try

                    Dim buff() As Byte = _serverOut.Bytes.GetBytes()
                    _udpServer.Send(buff)

                    lblServer.Text = "Server: Sent " + buff.Length.ToString + " byte(s)"
                    serverUpdate = Now()

                Catch ex As Exception

                    MessageBox.Show("Error: " + ex.Message, Application.ProductName, _
                        MessageBoxButtons.OK, MessageBoxIcon.Error)

                End Try

            Else

                MessageBox.Show("Not connected.", Application.ProductName, _
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

            End If

        End If

    End Sub

    Private Sub timerUpdate_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles timerUpdate.Tick

        If clientText <> "" AndAlso clientText <> lblClient.Text Then

            If Now.Subtract(clientUpdate).Seconds > UpdateExpires Then _
                lblClient.Text = clientText

        End If

        If serverText <> "" AndAlso serverText <> lblServer.Text Then

            If Now.Subtract(serverUpdate).Seconds > UpdateExpires Then _
                lblServer.Text = serverText

        End If

    End Sub

    Private Sub btnClientLoadFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClientLoadFile.Click

        Try

            fileOpen.ShowDialog()

            If fileOpen.FileName <> "" Then

                _clientOut.Bytes.Clear()
                _clientOut.Bytes.AddRange(File.ReadAllBytes(fileOpen.FileName))
                hexClientOut.Refresh()

            End If

        Catch
        End Try

    End Sub

    Private Sub btnClientSaveFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClientSaveFile.Click

        Try

            fileSave.ShowDialog()

            If fileSave.FileName <> "" Then

                Try

                    File.WriteAllBytes(fileSave.FileName, _clientOut.Bytes.GetBytes())

                Catch ex As Exception

                    MessageBox.Show("Error: " + ex.Message, _
                        Application.ProductName, MessageBoxButtons.OK, _
                        MessageBoxIcon.Error)

                End Try

            End If

        Catch
        End Try

    End Sub

    Private Sub btnServerLoadFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnServerLoadFile.Click

        Try

            fileOpen.ShowDialog()

            If fileOpen.FileName <> "" Then

                _serverOut.Bytes.Clear()
                _serverOut.Bytes.AddRange(File.ReadAllBytes(fileOpen.FileName))
                hexServerOut.Refresh()

            End If

        Catch
        End Try

    End Sub

    Private Sub btnServerSaveFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnServerSaveFile.Click

        Try

            fileSave.ShowDialog()

            If fileSave.FileName <> "" Then

                Try

                    File.WriteAllBytes(fileSave.FileName, _serverOut.Bytes.GetBytes())

                Catch ex As Exception

                    MessageBox.Show("Error: " + ex.Message, _
                        Application.ProductName, MessageBoxButtons.OK, _
                        MessageBoxIcon.Error)

                End Try

            End If

        Catch
        End Try

    End Sub

End Class