Option Explicit On

Imports javaSpy.clsWindowObject

Public Class frmWindowProps

    Private _thisWnd As clsWindowObject = Nothing

    Public Sub New(ByVal objWindow As clsWindowObject)

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        _thisWnd = objWindow

    End Sub

    Private Sub _showWaitCursor(ByVal bShow As Boolean)

        ' show/hide the hourglass
        If bShow Then
            Me.Cursor = Cursors.WaitCursor
            Me.UseWaitCursor = True
        Else
            Me.UseWaitCursor = False
            Me.Cursor = Cursors.Arrow
        End If

    End Sub

    Private Sub frmProperties_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Me.Text += FormatHandle(_thisWnd.hWnd)

        ' general page
        txtHandle.Tag = 0
        txtHandle.Text = "&H" + Hex(_thisWnd.hWnd)
        txtParent.Tag = 0
        txtParent.Text = "&H" + Hex(_thisWnd.hWndParent)
        txtClass.Text = _thisWnd.ClassName
        txtAssembly.Text = _thisWnd.AssemblyName
        numLeft.Value = _thisWnd.Left
        numLeft.Tag = False
        numTop.Value = _thisWnd.Top
        numTop.Tag = False
        numWidth.Value = _thisWnd.Width
        numWidth.Tag = False
        numHeight.Value = _thisWnd.Height
        numHeight.Tag = False
        txtCaption.Text = _thisWnd.Text
        txtCaption.Tag = False

        ' appearance page
        Select Case _thisWnd.WindowState
            Case WindowStates.SW_NORMAL : radNormal.Checked = True
            Case WindowStates.SW_MINIMIZE : radMinimized.Checked = True
            Case WindowStates.SW_MAXIMIZE : radMaximized.Checked = True
        End Select

        If _thisWnd.Enabled Then
            radEnabledTrue.Checked = True
        Else
            radEnabledFalse.Checked = True
        End If

        If _thisWnd.AlwaysOnTop Then
            radAlwaysOnTopTrue.Checked = True
        Else
            radAlwaysOnTopFalse.Checked = True
        End If

        btnMenuSpy.Enabled = _thisWnd.HasMenu
        btnApply.Enabled = False

    End Sub

    Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click

        Me.Close()

    End Sub

    Private Sub btnApply_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnApply.Click

        If numLeft.Tag Then _thisWnd.Left = numLeft.Value
        If numTop.Tag Then _thisWnd.Top = numTop.Value
        If numWidth.Tag Then _thisWnd.Width = numWidth.Value
        If numHeight.Tag Then _thisWnd.Height = numHeight.Value
        If txtCaption.Tag Then _thisWnd.Text = txtCaption.Text

        If radNormal.Checked Then
            _thisWnd.WindowState = WindowStates.SW_NORMAL
        ElseIf radMinimized.Checked Then
            _thisWnd.WindowState = WindowStates.SW_MINIMIZE
        ElseIf radMaximized.Checked Then
            _thisWnd.WindowState = WindowStates.SW_MAXIMIZE
        End If

        If radEnabledTrue.Checked Then
            _thisWnd.Enabled = True
        Else
            _thisWnd.Enabled = False
        End If

        If radAlwaysOnTopTrue.Checked Then
            _thisWnd.AlwaysOnTop = True
        Else
            _thisWnd.AlwaysOnTop = False
        End If

        btnApply.Enabled = False

    End Sub

    Private Sub btnOk_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOk.Click

        If tabMain.Enabled Then _
            btnApply.PerformClick()

        Me.Close()

    End Sub

    Private Sub btnUnload_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnUnload.Click

        _thisWnd.Close()
        Me.Close()

    End Sub

    Private Sub btnClickButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClickButton.Click

        _thisWnd.ClickButton(ClickButtonMethods.BTN_DEFAULT)

    End Sub

    Private Sub btnShowWindow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnShowWindow.Click

        _thisWnd.WindowState = WindowStates.SW_SHOW

    End Sub

    Private Sub btnHideWindow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnHideWindow.Click

        _thisWnd.WindowState = WindowStates.SW_HIDE

    End Sub

    Private Sub btnRestoreWindow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRestoreWindow.Click

        _thisWnd.WindowState = WindowStates.SW_RESTORE

    End Sub

    Private Sub btnFlashWindow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFlashWindow.Click
        _thisWnd.Flash()
    End Sub

    Private Sub lblHandle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lblHandle.Click

        txtHandle.Tag += 1

        Select Case txtHandle.Tag

            Case 1 : txtHandle.Text = "&H" + Hex(_thisWnd.hWnd)
            Case 2 : txtHandle.Text = FormatHandle(_thisWnd.hWnd)
            Case 3

                txtHandle.Text = _thisWnd.hWnd.ToString
                txtHandle.Tag = 0

        End Select

    End Sub

    Private Sub lblParent_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lblParent.Click

        txtParent.Tag += 1

        Select Case txtParent.Tag

            Case 1 : txtParent.Text = "&H" + Hex(_thisWnd.hWndParent)
            Case 2 : txtParent.Text = FormatHandle(_thisWnd.hWndParent)
            Case 3

                txtParent.Text = _thisWnd.hWndParent.ToString
                txtParent.Tag = 0

        End Select

    End Sub

    Private Sub numLeft_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles numLeft.KeyPress

        numLeft_ValueChanged(sender, e)

    End Sub

    Private Sub numLeft_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numLeft.ValueChanged

        numLeft.Tag = True
        btnApply.Enabled = True

    End Sub

    Private Sub numTop_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles numTop.KeyPress

        numTop_ValueChanged(sender, e)

    End Sub

    Private Sub numTop_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numTop.ValueChanged

        numTop.Tag = True
        btnApply.Enabled = True

    End Sub

    Private Sub numWidth_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles numWidth.KeyPress

        numWidth_ValueChanged(sender, e)

    End Sub

    Private Sub numWidth_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numWidth.ValueChanged

        numWidth.Tag = True
        btnApply.Enabled = True

    End Sub

    Private Sub numHeight_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles numHeight.KeyPress

        numHeight_ValueChanged(sender, e)

    End Sub

    Private Sub numHeight_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numHeight.ValueChanged

        numHeight.Tag = True
        btnApply.Enabled = True

    End Sub

    Private Sub txtCaption_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtCaption.TextChanged

        txtCaption.Tag = True
        btnApply.Enabled = True

    End Sub

    Private Sub btnMenuSpy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMenuSpy.Click

        _showWaitCursor(True)

        With New frmMenuSpy(_thisWnd)
            .Show(Me)
        End With

        _showWaitCursor(False)

    End Sub

    Private Sub radNormal_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radNormal.CheckedChanged

        btnApply.Enabled = True

    End Sub

    Private Sub radMinimized_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radMinimized.CheckedChanged

        btnApply.Enabled = True

    End Sub

    Private Sub radMaximized_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radMaximized.CheckedChanged

        btnApply.Enabled = True

    End Sub

    Private Sub radEnabledTrue_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radEnabledTrue.CheckedChanged

        btnApply.Enabled = True

    End Sub

    Private Sub radEnabledFalse_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radEnabledFalse.CheckedChanged

        btnApply.Enabled = True

    End Sub

    Private Sub radAlwaysOnTopTrue_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radAlwaysOnTopTrue.CheckedChanged

        btnApply.Enabled = True

    End Sub

    Private Sub radAlwaysOnTopFalse_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radAlwaysOnTopFalse.CheckedChanged

        btnApply.Enabled = True

    End Sub

    Private Sub btnProcessProps_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnProcessProps.Click

        _showWaitCursor(True)

        With New frmProcessProps(_thisWnd)
            .Show(Me)
        End With

        _showWaitCursor(False)

    End Sub

    Private Sub btnSendKeys_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSendKeys.Click

        _showWaitCursor(True)

        _thisWnd.SendText(txtSendKeys.Text, (100 - ((trackInterval.Value / trackInterval.Maximum) * 100)))

        _showWaitCursor(False)

    End Sub

    Private Sub btnHelp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnHelp.Click

        MessageBox.Show(GetResourceString("txtSendKeys"), _
            Application.ProductName + " :: Window :: Send Keys", _
            MessageBoxButtons.OK, MessageBoxIcon.Information)

    End Sub

    Private Sub txtSendKeys_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtSendKeys.TextChanged

        If txtSendKeys.Text = "" Then
            btnSendKeys.Enabled = False
        Else
            btnSendKeys.Enabled = True
        End If

    End Sub

    Private Sub btnReload_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnReload.Click

        Me.Text = "javaSpy :: Window :: "

        _thisWnd = New clsWindowObject(_thisWnd.hWnd)

        If _thisWnd.IsWindow Then

            frmProperties_Load(sender, e)

        Else

            MessageBox.Show("The window is no longer valid.", _
                Application.ProductName, MessageBoxButtons.OK, _
                MessageBoxIcon.Exclamation)

            Me.Text += "(invalid)"
            tabMain.Enabled = False
            btnApply.Enabled = False

        End If

    End Sub

End Class